package;

import haxe.Json;
import kha.Assets;
import kha.Color;
import kha.Font;
import kha.Framebuffer;
import kha.Scheduler;
import kha.Shaders;
import kha.System;
import kha.graphics4.BlendingFactor;
import kha.graphics4.CompareMode;
import kha.graphics4.ConstantLocation;
import kha.graphics4.IndexBuffer;
import kha.graphics4.MipMapFilter;
import kha.graphics4.PipelineState;
import kha.graphics4.TextureAddressing;
import kha.graphics4.TextureFilter;
import kha.graphics4.TextureUnit;
import kha.graphics4.Usage;
import kha.graphics4.VertexBuffer;
import kha.graphics4.VertexData;
import kha.graphics4.VertexStructure;
import kha.math.FastMatrix3;
import kha.math.FastMatrix4;
import kha.math.FastVector4;
import zui.Id;
import zui.Zui;

class Project {
	
 	
	var shaderPipeline: PipelineState = null;
	var structure: VertexStructure = null;
	var projectionLocation: ConstantLocation;
	var textureLocation: TextureUnit;
	var vertexBuffer:VertexBuffer;
	var indexBuffer:IndexBuffer;
	var ui:Zui;
	var angle:Float = 0;
	
	public function new() {
		
		initShaders(); 
		
		vertexBuffer = new VertexBuffer(100, structure, Usage.StaticUsage);
		indexBuffer = new IndexBuffer(100, Usage.StaticUsage);
		
		var vertices = vertexBuffer.lock();
		var indices = indexBuffer.lock();
		
		function setVertex(n, x, y, tx, ty) {
			vertices.set(n * 9 + 0, x);
			vertices.set(n * 9 + 1, y);
			vertices.set(n * 9 + 2, -900);
			vertices.set(n * 9 + 3, tx);
			vertices.set(n * 9 + 4, ty);
			vertices.set(n * 9 + 5, 1);
			vertices.set(n * 9 + 6, 1);
			vertices.set(n * 9 + 7, 1);
			vertices.set(n * 9 + 8, 1);
		}                             
		                              
		setVertex(0, 0, 0, 0, 0);     
		setVertex(1, 100, 0, 1, 0);
		setVertex(2, 100, 100, 1, 1);
		setVertex(3, 0, 100, 0, 1);
		
		indices[0] = 0;
		indices[1] = 1;
		indices[2] = 2;
		indices[3] = 0;
		indices[4] = 2;
		indices[5] = 3;
		
		vertexBuffer.unlock();
		indexBuffer.unlock();
		
		Assets.loadEverything(function() {
			ui = new Zui(Assets.fonts.arial);
			System.notifyOnRender(render);
			Scheduler.addTimeTask(update, 0, 1 / 60);
		});
		
		var projection = FastMatrix4.orthogonalProjection(0, 800, 600, 0, 0.1, 1000);
		
		for(z in -10...10) {
			var v = projection.multvec(new FastVector4(100, 100, z * 100));
			trace(z, v.x, v.y, v.z, v.w);
		}
	}
	

	function update(): Void {
		
	}

	function render(framebuffer: Framebuffer): Void {
		
		var projection = FastMatrix4.orthogonalProjection( -framebuffer.width / 4, framebuffer.width / 4, framebuffer.height / 4, -framebuffer.height / 4, 0.1, 1000);
		var model = FastMatrix4.rotation(0, angle * Math.PI / 180, 0).multmat(FastMatrix4.translation( -50, -50, 0));
		//var model3 = FastMatrix3.rotation(angle * Math.PI / 180);
		//var model = new FastMatrix4(
			//model3._00, model3._10, 0, model3._20,
			//model3._01, model3._11, 0, model3._21,
			//0, 0, 1, 0,
			//model3._02, model3._12, 0, model3._22
		//);
		
		var matrix = projection.multmat(model);
		var v = matrix.multvec(new FastVector4(100, 100, -5, 1));
		
		var g4 = framebuffer.g4;
		g4.begin();
		g4.clear(Color.Black);
		
		g4.setVertexBuffer(vertexBuffer);
		g4.setIndexBuffer(indexBuffer);
		g4.setPipeline(shaderPipeline);
		g4.setTexture(textureLocation, Assets.images.bitmap);
		g4.setMatrix(projectionLocation, matrix);
		g4.setTextureParameters(textureLocation, TextureAddressing.Clamp, TextureAddressing.Clamp, TextureFilter.LinearFilter, TextureFilter.LinearFilter, MipMapFilter.NoMipFilter);
		g4.drawIndexedVertices(0, 6);
		g4.setTexture(textureLocation, null);
		g4.end();
		
        ui.begin(framebuffer.g2);
        if (ui.window(Id.window(), 0, 0, 200, framebuffer.height, Zui.LAYOUT_VERTICAL)) {
			angle = ui.slider(Id.slider(), "angle", 0, 360, false, 1, angle);
        }
        ui.end();
			
	}
	
	private function initShaders(): Void {
		if (shaderPipeline != null) return;
		
		shaderPipeline = new PipelineState();
		shaderPipeline.fragmentShader = Shaders.painter_image_frag;
		shaderPipeline.vertexShader = Shaders.painter_image_vert;

		structure = new VertexStructure();
		structure.add("vertexPosition", VertexData.Float3);
		structure.add("texPosition", VertexData.Float2);
		structure.add("vertexColor", VertexData.Float4);
		shaderPipeline.inputLayout = [structure];
		
		shaderPipeline.blendSource = BlendingFactor.BlendOne;
		shaderPipeline.blendDestination = BlendingFactor.InverseSourceAlpha;
		shaderPipeline.alphaBlendSource = BlendingFactor.SourceAlpha;
		shaderPipeline.alphaBlendDestination = BlendingFactor.InverseSourceAlpha;
		
		shaderPipeline.compile();
		
		projectionLocation = shaderPipeline.getConstantLocation("projectionMatrix");
		textureLocation = shaderPipeline.getTextureUnit("tex");
	}
	
}
